package Slapd::Backend::Cyrus::Mail::Test;

=head1 NAME

Slapd::Backend::Cyrus::Mail::Test - tests for Slapd::Backend::Cyrus::Mail

=cut

use utf8;
use strict;
use warnings;
use base 'Slapd::Backend::Cyrus::Test';
use Cyrus::IMAP::Admin;
use Net::LDAP;
use Net::LDAP::Constant qw( LDAP_TYPE_OR_VALUE_EXISTS LDAP_COMPARE_TRUE LDAP_COMPARE_FALSE );
use Test::More;

use Slapd::Backend::Cyrus::Mail;

sub startup_mail_config : Test(startup)
{
	my $self = shift;

	$self->{folder_class} = $1 if $self->{config_content} =~ /^cyrusFolder\s+"?\(objectClass=(.+?)\)"?$/m;
	$self->{acl_class} = $1 if $self->{config_content} =~ /^cyrusAcl\s+"?\(objectClass=(.+?)\)"?$/m;

	return;
}

sub add_folder_entry
{
	my( $self, $domain_dn, @attrs ) = @_;
	my %attrs = @attrs;

	my $dn = "cn=$attrs{cn},$domain_dn";
	my $mesg = $self->{ldap}->add( $dn, attr => [ objectClass => $self->{folder_class}, @attrs ] );
	diag( $mesg->error() ) if $mesg->is_error();

	return $dn;
}

sub add_acl_entry
{
	my( $self, $folder_dn, @attrs ) = @_;
	my %attrs = @attrs;

	my $dn = "cn=$attrs{cn},$folder_dn";
	my $mesg = $self->{ldap}->add( $dn, attr => [ objectClass => $self->{acl_class}, @attrs ] );
	diag( $mesg->error() ) if $mesg->is_error();

	return $dn;
}

sub check_mailbox
{
	my( $self, $domain_attrs, $folder_attrs ) = @_;
	return 1 if $self->cyradm->listmailbox( 'shared/' . $folder_attrs->{cn} . '@' . $domain_attrs->{dc} );
	return;
}

sub check_mailbox_acl
{
	my( $self, $domain_attrs, $folder_attrs, $acl_attrs ) = @_;
	my %acl = $self->cyradm->listaclmailbox( 'shared/' . $folder_attrs->{cn} . '@' . $domain_attrs->{dc} );
	my @users;
	foreach my $user ( keys %acl )
	{
		next if $user ~~ [ Slapd::Backend::Cyrus::Mail->skip_users ];
		push @users, $user if Slapd::Backend::Cyrus::Mail->_check_acl( $acl_attrs->{cn}, $acl{$user} );
	}
	return [ sort @{$acl_attrs->{memberUid}} ] ~~ [ sort @users ];
}

sub search : Test(4)
{
	my $self = shift;

	my %domain_attrs = ( dc => 'search.test' );
	my %folder_attrs = ( cn => 'test-list', description => "Тестовый список 1" );
	my %acl_attrs = ( cn => 'post', memberUid => [$self->{test_user}->[1], $self->{test_user}->[2]] );
	my %acl2_attrs = ( cn => 'delete' );
	my $domain_dn = $self->add_domain_entry( %domain_attrs );
	my $folder_dn = $self->add_folder_entry( $domain_dn, %folder_attrs );
	my $acl_dn = $self->add_acl_entry( $folder_dn, %acl_attrs );
	my $acl2_dn = $self->add_acl_entry( $folder_dn, %acl2_attrs );

	my $mesg = $self->{ldap}->search(
		base   => $self->{mail_base},
		filter => '(dc=search.test)',
		raw    => qr/;binary/,
	);
	ok(
		!$mesg->is_error() && $mesg->count() == 1
		&& $mesg->entry(0)->get_value('dc') eq 'search.test',
		"Search domain by dc"
	) or diag( $mesg->error() );

	$mesg = $self->{ldap}->search(
		base   => $self->{mail_base},
		filter => '(description=Тестовый список 1)',
		raw    => qr/;binary/,
	);
	ok(
		!$mesg->is_error() && $mesg->count() == 1
		&& $mesg->entry(0)->get_value('cn') eq 'test-list',
		"Search folder by description"
	) or diag( $mesg->error() );

	$mesg = $self->{ldap}->search(
		base   => $folder_dn,
		filter => "(memberUid=$self->{test_user}->[1])",
		raw    => qr/;binary/,
	);
	ok(
		!$mesg->is_error() && $mesg->count() == 1
		&& $mesg->entry(0)->dn() eq $acl_dn,
		"Search acl by memberUid"
	) or diag( $mesg->error() );

	$mesg = $self->{ldap}->search(
		base   => $folder_dn,
		filter => "(cn=$acl2_attrs{cn})",
		raw    => qr/;binary/,
	);
	ok(
		!$mesg->is_error() && $mesg->count() == 1
		&& $mesg->entry(0)->dn() eq $acl2_dn,
		"Search acl with empty memberUid by cn"
	) or diag( $mesg->error() );

	return;
}

sub add_domain : Test(3)
{
	my $self = shift;

	my %user_attrs = ( uid => $self->{test_user}->[1], mailLocalAddress => 'trampam@add.test' );
	my $user_dn = $self->add_user_entry( %user_attrs );

	my %domain_attrs = ( dc => 'add.test', objectClass => $self->{domain_class} );
	my $domain_dn = "dc=$domain_attrs{dc},$self->{mail_base}";

	my $mesg = $self->{ldap}->add( $domain_dn, attr => [ %domain_attrs ] );
	ok( !$mesg->is_error(), "Add new domain" ) or diag( $mesg->error() );

	ok( $self->check_search( $domain_dn, %domain_attrs ), "Add new domain (search)" );
	ok( $self->check_user_mailbox( \%domain_attrs, \%user_attrs ), "Add new domain (cyradm user mailbox)" );

	return;
}

sub add_folder : Test(3)
{
	my $self = shift;

	my %domain_attrs = ( dc => 'add-folder.test' );
	my $domain_dn = $self->add_domain_entry( %domain_attrs );

	my $folder_dn = "cn=test-list,$domain_dn";
	my %folder_attrs = ( cn => 'test-list', objectClass => $self->{folder_class} );

	my $mesg = $self->{ldap}->add( $folder_dn, attr => [ %folder_attrs ] );
	ok( !$mesg->is_error(), "Add new folder" ) or diag( $mesg->error() );

	ok( $self->check_search( $folder_dn, %folder_attrs ), "Add new folder (search)" );
	ok( $self->check_mailbox( \%domain_attrs, \%folder_attrs ), "Add new folder (cyradm)" );

	return;
}

sub add_subfolder : Test(3)
{
	my $self = shift;

	my %domain_attrs = ( dc => 'modify-acl-user.test' );
	my %folder_attrs = ( cn => 'test-list' );
	my %acl_attrs = ( cn => 'post', memberUid => [$self->{test_user}->[1], $self->{test_user}->[2]] );
	my $domain_dn = $self->add_domain_entry( %domain_attrs );
	my $folder_dn = $self->add_folder_entry( $domain_dn, %folder_attrs );
	my $acl_dn = $self->add_acl_entry( $folder_dn, %acl_attrs );

	my %subfolder_attrs = ( cn => 'test-sublist' );
	my %subacl_attrs = ( cn => 'delete', memberUid => [] );
	my $subfolder_dn = $self->add_folder_entry( $folder_dn, %subfolder_attrs );
	my $subacl_dn = $self->add_acl_entry( $folder_dn, %acl_attrs );

	ok( $self->check_search( $subfolder_dn, %subfolder_attrs ), "Add new subfolder (search)" );
	ok( $self->check_mailbox( \%domain_attrs, { cn => "$folder_attrs{cn}/$subfolder_attrs{cn}" } ), "Add new subfolder (cyradm)" );
	ok( !$self->check_mailbox_acl( \%domain_attrs, { cn => "$folder_attrs{cn}/$subfolder_attrs{cn}" }, \%acl_attrs ), "Add new subfolder (clear inherited from parent acls) (cyradm)" );

	return;
}

sub add_acl : Test(8)
{
	my $self = shift;

	my %domain_attrs = ( dc => 'add-acl.test' );
	my %folder_attrs = ( cn => 'test-list' );
	my $domain_dn = $self->add_domain_entry( %domain_attrs );
	my $folder_dn = $self->add_folder_entry( $domain_dn, %folder_attrs );

	my $acl_dn = "cn=post,$folder_dn";
	my %acl_attrs = (
		cn          => 'post',
		objectClass => 'applicationProcess',
		memberUid   => [$self->{test_user}->[1]],
	);

	my $mesg = $self->{ldap}->add( $acl_dn, attr => [ %acl_attrs ] );
	ok( !$mesg->is_error(), "Add new acl" ) or diag( $mesg->error() );

	ok( $self->check_search( $acl_dn, %acl_attrs ), "Add new acl (search)" );
	ok( $self->check_mailbox_acl( \%domain_attrs, \%folder_attrs, \%acl_attrs ), "Add new acl (cyradm)" );

	my $acl2_dn = "cn=delete,$folder_dn";
	my %acl2_attrs = (
		cn          => 'delete',
		objectClass => 'applicationProcess',
		memberUid   => [$self->{test_user}->[1]],
	);

	$mesg = $self->{ldap}->add( $acl2_dn, attr => [ %acl2_attrs ] );
	ok( !$mesg->is_error(), "Add new acl (second)" ) or diag( $mesg->error() );

	ok( $self->check_search( $acl2_dn, %acl2_attrs ), "Add new acl (second) (search)" );
	ok( $self->check_search( $acl_dn, %acl_attrs ), "Add new acl (search previous)" );
	ok( $self->check_mailbox_acl( \%domain_attrs, \%folder_attrs, \%acl2_attrs ), "Add new acl (second) (cyradm)" );
	ok( $self->check_mailbox_acl( \%domain_attrs, \%folder_attrs, \%acl_attrs ), "Add new acl (cyradm previous)" );

	return;
}

sub modify_acl_user : Test(13)
{
	my $self = shift;

	my %domain_attrs = ( dc => 'modify-acl-user.test' );
	my %folder_attrs = ( cn => 'test-list' );
	my %acl_attrs = ( cn => 'post', memberUid => [$self->{test_user}->[1], $self->{test_user}->[2]] );
	my $domain_dn = $self->add_domain_entry( %domain_attrs );
	my $folder_dn = $self->add_folder_entry( $domain_dn, %folder_attrs );
	my $acl_dn = $self->add_acl_entry( $folder_dn, %acl_attrs );

	my %sec_acl_attrs = ( cn => 'delete', memberUid => [$self->{test_user}->[3]] );
	my $sec_acl_dn = $self->add_acl_entry( $folder_dn, %sec_acl_attrs );

	push @{$acl_attrs{memberUid}}, $self->{test_user}->[3];
	my $mesg = $self->{ldap}->modify( $acl_dn, add => { memberUid => $self->{test_user}->[3] } );
	ok( !$mesg->is_error(), "Modify acl (add memberUid)" ) or diag( $mesg->error() );

	ok( $self->check_search( $acl_dn, %acl_attrs ), "Modify acl (add memberUid) (search)" );
	ok( $self->check_search( $sec_acl_dn, %sec_acl_attrs ), "Modify acl (add memberUid) (search secondary)" );
	ok( $self->check_mailbox_acl( \%domain_attrs, \%folder_attrs, \%acl_attrs ), "Modify acl (add memberUid) (cyradm)" );
	ok( $self->check_mailbox_acl( \%domain_attrs, \%folder_attrs, \%sec_acl_attrs ), "Modify acl (add memberUid) (cyradm secondary)" );

	pop @{$acl_attrs{memberUid}};
	$mesg = $self->{ldap}->modify( $acl_dn, delete => { memberUid => $self->{test_user}->[3] } );
	ok( !$mesg->is_error(), "Modify acl (delete memberUid)" ) or diag( $mesg->error() );

	ok( $self->check_search( $acl_dn, %acl_attrs ), "Modify acl (delete memberUid) (search)" );
	ok( $self->check_search( $sec_acl_dn, %sec_acl_attrs ), "Modify acl (delete memberUid) (search secondary)" );
	ok( $self->check_mailbox_acl( \%domain_attrs, \%folder_attrs, \%acl_attrs ), "Modify acl (delete memberUid) (cyradm)" );
	ok( $self->check_mailbox_acl( \%domain_attrs, \%folder_attrs, \%sec_acl_attrs ), "Modify acl (delete memberUid) (cyradm secondary)" );

	pop @{$acl_attrs{memberUid}};
	push @{$acl_attrs{memberUid}}, $self->{test_user}->[4];
	$mesg = $self->{ldap}->modify( $acl_dn, replace => { memberUid => [$self->{test_user}->[1], $self->{test_user}->[4]] } );
	ok( !$mesg->is_error(), "Modify acl (replace memberUid)" ) or diag( $mesg->error() );

	ok( $self->check_search( $acl_dn, %acl_attrs ), "Modify acl (replace memberUid) (search)" );
	ok( $self->check_mailbox_acl( \%domain_attrs, \%folder_attrs, \%acl_attrs ), "Modify acl (replace memberUid) (cyradm)" );

	return;
}

sub modify_inout_filter : Test(6)
{
	my $self = shift;

	my %domain_attrs = ( dc => 'modify-inout-filter.test' );
	my %folder_attrs = ( cn => 'test-list' );
	my %acl_attrs = ( cn => 'post', memberUid => [$self->{test_user}->[1], $self->{test_user}->[2]] );
	my $domain_dn = $self->add_domain_entry( %domain_attrs );
	my $folder_dn = $self->add_folder_entry( $domain_dn, %folder_attrs );
	my $acl_dn = $self->add_acl_entry( $folder_dn, %acl_attrs );

	my $mesg = $self->{ldap}->modify( $acl_dn, replace => { objectClass => 'top' } );
	ok( !$mesg->is_error(), "Modify acl (exclude from filter)" ) or diag( $mesg->error() );

	ok( $self->check_search( $acl_dn, %acl_attrs ), "Modify acl (exclude from filter) (search)" );
	ok( !$self->check_mailbox_acl( \%domain_attrs, \%folder_attrs, \%acl_attrs ), "Modify acl (exclude from filter) (cyradm)" );

	$mesg = $self->{ldap}->modify( $acl_dn, replace => { objectClass => $self->{acl_class} } );
	ok( !$mesg->is_error(), "Modify acl (include to filter)" ) or diag( $mesg->error() );

	ok( $self->check_search( $acl_dn, %acl_attrs ), "Modify acl (include to filter) (search)" );
	ok( $self->check_mailbox_acl( \%domain_attrs, \%folder_attrs, \%acl_attrs ), "Modify acl (include to filter) (cyradm)" );

	return;
}

sub modrdn_acl : Test(5)
{
	my $self = shift;

	my %domain_attrs = ( dc => 'modrdn-acl.test' );
	my %folder_attrs = ( cn => 'test-list' );
	my %acl_attrs = ( cn => 'post', memberUid => [$self->{test_user}->[1], $self->{test_user}->[2]] );
	my $domain_dn = $self->add_domain_entry( %domain_attrs );
	my $folder_dn = $self->add_folder_entry( $domain_dn, %folder_attrs );
	my $acl_dn = $self->add_acl_entry( $folder_dn, %acl_attrs );

	my $mesg = $self->{ldap}->moddn(
		$acl_dn,
		newrdn => "cn=delete",
		deleteoldrdn => 1,
	);
	ok( !$mesg->is_error(), "Modrdn acl" ) or diag( $mesg->error() );

	ok( !$self->check_search( $acl_dn ), "Modrdn acl (search old)" );
	ok( !$self->check_mailbox_acl( \%domain_attrs, \%folder_attrs, \%acl_attrs ), "Modrdn acl (cyradm old)" );
	$acl_attrs{cn} = 'delete';
	$acl_dn =~ s/cn=post/cn=delete/;
	ok( $self->check_search( $acl_dn, %acl_attrs ), "Modrdn acl (search new)" );
	ok( $self->check_mailbox_acl( \%domain_attrs, \%folder_attrs, \%acl_attrs ), "Modrdn acl (cyradm new)" );

	return;
}

sub modrdn_domain : Test(5)
{
	my $self = shift;

	my %domain_attrs = ( dc => 'modrdn-domain.test' );
	my %folder_attrs = ( cn => 'test-list' );
	my %acl_attrs = ( cn => 'post', memberUid => [$self->{test_user}->[1], $self->{test_user}->[2]] );
	my $domain_dn = $self->add_domain_entry( %domain_attrs );
	my $folder_dn = $self->add_folder_entry( $domain_dn, %folder_attrs );
	my $acl_dn = $self->add_acl_entry( $folder_dn, %acl_attrs );

	my $mesg = $self->{ldap}->moddn(
		$domain_dn,
		newrdn => "dc=modrdn-domain-new.test",
		deleteoldrdn => 1,
	);
	ok( !$mesg->is_error(), "Modrdn domain" ) or diag( $mesg->error() );

	ok( !$self->check_search( $acl_dn ), "Modrdn domain (search old acl)" );
	ok( !$self->check_mailbox_acl( \%domain_attrs, \%folder_attrs, \%acl_attrs ), "Modrdn domain (cyradm old acl)" );
	$acl_dn =~ s/$domain_attrs{dc}/modrdn-domain-new.test/;
	$domain_attrs{dc} = 'modrdn-domain-new.test';
	ok( $self->check_search( $acl_dn, %acl_attrs ), "Modrdn domain (search new acl)" );
	ok( $self->check_mailbox_acl( \%domain_attrs, \%folder_attrs, \%acl_attrs ), "Modrdn domain (cyradm new acl)" );

	return;
}

sub delete_acl : Test(5)
{
	my $self = shift;

	my %domain_attrs = ( dc => 'delete-acl.test' );
	my %folder_attrs = ( cn => 'test-list' );
	my %acl_attrs = ( cn => 'post', memberUid => [$self->{test_user}->[1], $self->{test_user}->[2]] );
	my $domain_dn = $self->add_domain_entry( %domain_attrs );
	my $folder_dn = $self->add_folder_entry( $domain_dn, %folder_attrs );
	my $acl_dn = $self->add_acl_entry( $folder_dn, %acl_attrs );

	my %sec_acl_attrs = ( cn => 'delete', memberUid => [$self->{test_user}->[2]] );
	my $sec_acl_dn = $self->add_acl_entry( $folder_dn, %sec_acl_attrs );

	my $mesg = $self->{ldap}->delete( $acl_dn );
	ok( !$mesg->is_error(), "Delete acl" ) or diag( $mesg->error() );

	ok( !$self->check_search( $acl_dn ), "Delete acl (search)" );
	ok( $self->check_search( $sec_acl_dn, %sec_acl_attrs ), "Delete acl (search secondary)" );

	ok( !$self->check_mailbox_acl( \%domain_attrs, \%folder_attrs, \%acl_attrs ), "Delete acl (cyradm)" );
	ok( $self->check_mailbox_acl( \%domain_attrs, \%folder_attrs, \%sec_acl_attrs ), "Delete acl (cyradm secondary)" );

	return;
}

sub compare : Test(10)
{
	my $self = shift;

	my %domain_attrs = ( dc => 'compare.test', description => 'Тестовое сравнение' );
	my %folder_attrs = ( cn => 'test-list' );
	my %acl_attrs = ( cn => 'write', memberUid => [$self->{test_user}->[1], $self->{test_user}->[2]] );
	my $domain_dn = $self->add_domain_entry( %domain_attrs );
	my $folder_dn = $self->add_folder_entry( $domain_dn, %folder_attrs );
	my $acl_dn = $self->add_acl_entry( $folder_dn, %acl_attrs );

	my $mesg = $self->{ldap}->compare( $domain_dn, attr => 'dc', value => $domain_attrs{dc} );
	ok( $mesg->code() eq LDAP_COMPARE_TRUE, "Compare domain by dc (true)" ) or diag( $mesg->error() );

	$mesg = $self->{ldap}->compare( $domain_dn, attr => 'dc', value => 'compare-x.test' );
	ok( $mesg->code() eq LDAP_COMPARE_FALSE, "Compare domain by dc (false)" ) or diag( $mesg->error() );

	$mesg = $self->{ldap}->compare( $domain_dn, attr => 'description', value => $domain_attrs{description} );
	ok( $mesg->code() eq LDAP_COMPARE_TRUE, "Compare by description (true)" ) or diag( $mesg->error() );

	$mesg = $self->{ldap}->compare( $domain_dn, attr => 'description', value => 'Не то' );
	ok( $mesg->code() eq LDAP_COMPARE_FALSE, "Compare by description (false)" ) or diag( $mesg->error() );

	$mesg = $self->{ldap}->compare( $folder_dn, attr => 'cn', value => $folder_attrs{cn} );
	ok( $mesg->code() eq LDAP_COMPARE_TRUE, "Compare folder by cn (true)" ) or diag( $mesg->error() );

	$mesg = $self->{ldap}->compare( $folder_dn, attr => 'cn', value => 'test-list-x' );
	ok( $mesg->code() eq LDAP_COMPARE_FALSE, "Compare folder by cn (false)" ) or diag( $mesg->error() );

	$mesg = $self->{ldap}->compare( $acl_dn, attr => 'cn', value => $acl_attrs{cn} );
	ok( $mesg->code() eq LDAP_COMPARE_TRUE, "Compare acl by cn (true)" ) or diag( $mesg->error() );

	$mesg = $self->{ldap}->compare( $acl_dn, attr => 'cn', value => 'read' );
	ok( $mesg->code() eq LDAP_COMPARE_FALSE, "Compare acl by cn (false)" ) or diag( $mesg->error() );

	$mesg = $self->{ldap}->compare( $acl_dn, attr => 'memberUid', value => $self->{test_user}->[1] );
	ok( $mesg->code() eq LDAP_COMPARE_TRUE, "Compare acl by memberUid (true)" ) or diag( $mesg->error() );

	$mesg = $self->{ldap}->compare( $acl_dn, attr => 'memberUid', value => $self->{test_user}->[3] );
	ok( $mesg->code() eq LDAP_COMPARE_FALSE, "Compare acl by memberUid (false)" ) or diag( $mesg->error() );

	return;
}

1;

=head1 SEE ALSO

L<Slapd::Backend::Cyrus::Mail>, L<cyradm>, L<slapd-perl>, L<slapd>

=head1 AUTHOR

Aleksey Mashanov <aleksey.mashanov@gmail.com>

=head1 LICENSE

This library is free software, you can redistribute it and/or modify it under
the same terms as Perl itself.

=cut
