package Slapd::Backend::Cyrus::Test;

=head1 NAME

Slapd::Backend::Cyrus::Test - base class for slapd-cyrus tests

=cut

use utf8;
use strict;
use warnings;
use base 'Slapd::Backend::Test', 'Class::Accessor::Lvalue::Fast';
use Cyrus::IMAP::Admin;
use Test::More;

__PACKAGE__->mk_accessors('cyradm');

sub startup_cyrus : Test(startup)
{
	my $self = shift;

	my( $cyrus_server ) = $self->{config_content} =~ /^cyrusServer\s+(.+)$/m;
	my( $cyrus_user, $cyrus_password ) = $self->{config_content} =~ /^cyrusAuth\s+(.+) (.+)$/m;
	$self->cyradm = new Cyrus::IMAP::Admin( $cyrus_server )
		or die "Can't connect to server $cyrus_server\n";
	$self->cyradm->authenticate(
		-user     => $cyrus_user,
		-password => $cyrus_password,
	) or die $self->cyradm->error();
	$self->{cyrus_user} = $cyrus_user;

	return;
}

sub shutdown_cyrus : Test(shutdown)
{
	my $self = shift;
	for( 0 .. 1 ) # Cyrus can be configured for delayed delete
	{
		foreach my $mb_info ( $self->cyradm->listmailbox( '*' ) ) # Here must be '*.test', but it not works on test host
		{
			warn "@$mb_info";
			$self->cyradm->setaclmailbox( $mb_info->[0], $self->{cyrus_user}, 'x' ) or diag( $self->cyradm->error() );
			$self->cyradm->deletemailbox( $mb_info->[0] ) or diag( $self->cyradm->error() );
		}
	}
	return;
}

sub startup_cyrus_config : Test(startup)
{
	my $self = shift;

	$self->{user_base} = $1 if $self->{config_content} =~ /^suffix\s+"?(.+?)"?$/mg;
	$self->{mail_base} = $1 if $self->{config_content} =~ /^suffix\s+"?(.+?)"?$/mg;

	$self->{user_class} = $1 if $self->{config_content} =~ /^cyrusUser\s+"?\(objectClass=(.+?)\)"?$/m;
	$self->{domain_class} = $1 if $self->{config_content} =~ /^cyrusDomain\s+"?\(objectClass=(.+?)\)"?$/m;

	return;
}

sub add_domain_entry
{
	my( $self, %attrs ) = @_;

	my $dn = "dc=$attrs{dc},$self->{mail_base}";
	my $mesg = $self->{ldap}->add( $dn, attr => [ objectClass => $self->{domain_class}, %attrs ] );
	diag( $mesg->error() ) if $mesg->is_error();

	return $dn;
}

sub add_user_entry
{
	my( $self, %attrs ) = @_;

	my $dn = "uid=$attrs{uid},$self->{user_base}";
	my $mesg = $self->{ldap}->add( $dn, attr => [ objectClass => $self->{user_class}, %attrs ] );
	diag( $mesg->error() ) if $mesg->is_error();

	return $dn;
}

sub check_search
{
	my( $self, $dn, %attrs ) = @_;

	my $mesg = $self->{ldap}->search(
		base   => $dn,
		filter => '(objectClass=*)',
		scope  => 'base',
		raw    => qr/;binary/,
	);
	if( $mesg->is_error() )
	{
		diag( $mesg->error() );
		return;
	}

	return unless $mesg->count() == 1;
	my $entry = $mesg->entry(0);
	foreach my $attr ( keys %attrs )
	{
		if( ref $attrs{$attr} eq 'ARRAY' )
		{
			my @values = $entry->get_value($attr);
			return unless [ sort @values ] ~~ [ sort @{$attrs{$attr}} ];
		}
		else
		{
			return unless defined $attrs{$attr} ? $entry->get_value($attr) eq $attrs{$attr} : !$entry->exists($attr);
		}
	}

	return 1;
}

sub check_user_mailbox
{
	my( $self, $domain_attrs, $user_attrs ) = @_;
	return 1 if $self->cyradm->listmailbox( 'user/' . $user_attrs->{uid} . '@' . $domain_attrs->{dc} );
	return;
}

# Cyrus can be configured to use ldap database for user validation.
# So we out to connect to it and load short list of valid uids.
sub startup_valid_users : Test(startup)
{
	my $self = shift;

	my( $host, $binddn, $bindpw );
	my $imap = new IO::File( "/usr/local/etc/imapd.conf", '<' );
	while( defined( my $line = $imap->getline() ) )
	{
		$host = $1 if $line =~ /^ldap_uri:\s*(.+)$/;
		$binddn = $1 if $line =~ /^ldap_bind_dn:\s*(.+)$/;
		$bindpw = $1 if $line =~ /^ldap_password:\s*(.+)$/;
	}
	$imap->close();

	if( $host )
	{
		my $ldap = new Net::LDAP( $host ) or die $@;
		my $mesg = $ldap->bind( $binddn, password => $bindpw );
		die $mesg->error() if $mesg->code();
		$mesg = $ldap->search(
			base   => $self->{user_base},
			filter => "(objectClass=$self->{user_class})",
			raw    => qr/;binary/,
		);
		die $mesg->error() if $mesg->code();
		my $i = 0;
		foreach my $entry ( $mesg->entries() )
		{
			push @{$self->{test_user}}, $entry->get_value('uid');
			$i++;
			last if $i == 5;
		}
		die "At least 4 users must be present" if $i < 5;
	}
	else
	{
		$self->{test_user} = [ map "test$_", ( 0 .. 4 ) ];
	}

	return;
}

1;

=head1 SEE ALSO

L<cyradm>, L<slapd-perl>, L<slapd>

=head1 AUTHOR

Aleksey Mashanov <aleksey.mashanov@gmail.com>

=head1 LICENSE

This library is free software, you can redistribute it and/or modify it under
the same terms as Perl itself.

=cut
