package Slapd::Backend::Cyrus::User::Test;

=head1 NAME

Slapd::Backend::Cyrus::User::Test - tests for Slapd::Backend::Cyrus::User

=cut

use utf8;
use strict;
use warnings;
use base 'Slapd::Backend::Cyrus::Test';
use Cyrus::IMAP::Admin;
use Net::LDAP;
use Net::LDAP::Constant qw( LDAP_TYPE_OR_VALUE_EXISTS LDAP_COMPARE_TRUE LDAP_COMPARE_FALSE );
use Test::More;

sub search : Test(3)
{
	my $self = shift;

	my $domain_dn = $self->add_domain_entry( dc => 'search-user.test' );

	my %user_attrs = ( uid => $self->{test_user}->[1], mailLocalAddress => ['bla@search-user.test', 'bla@search-user-2.test'] );
	my %user2_attrs = ( uid => $self->{test_user}->[2] );
	my $user_dn = $self->add_user_entry( %user_attrs );
	my $user2_dn = $self->add_user_entry( %user2_attrs );

	my $mesg = $self->{ldap}->search(
		base   => $self->{user_base},
		filter => "(uid=$user_attrs{uid})",
		raw    => qr/;binary/,
	);
	ok(
		!$mesg->is_error() && $mesg->count() == 1
		&& $mesg->entry(0)->get_value('uid') eq $user_attrs{uid},
		"Search user by uid"
	) or diag( $mesg->error() );

	$mesg = $self->{ldap}->search(
		base   => $self->{user_base},
		filter => "(mailLocalAddress=bla\@search-user.test)",
		raw    => qr/;binary/,
	);
	ok(
		!$mesg->is_error() && $mesg->count() == 1
		&& $mesg->entry(0)->dn() eq $user_dn,
		"Search user by mailLocalAddress"
	) or diag( $mesg->error() );

	$mesg = $self->{ldap}->search(
		base   => $self->{user_base},
		filter => "(uid=$user2_attrs{uid})",
		raw    => qr/;binary/,
	);
	ok(
		!$mesg->is_error() && $mesg->count() == 1
		&& $mesg->entry(0)->get_value('uid') eq $user2_attrs{uid},
		"Search user with empty mailLocalAddress by uid"
	) or diag( $mesg->error() );

	return;
}

sub add_user : Test(3)
{
	my $self = shift;

	my %domain_attrs = ( dc => 'add-user.test' );
	my $domain_dn = $self->add_domain_entry( %domain_attrs );

	my %user_attrs = (
		objectClass      => $self->{user_class},
		uid              => $self->{test_user}->[1],
		mailLocalAddress => ['bla@add-user.test'],
	);
	my $user_dn = "uid=$user_attrs{uid},$self->{user_base}";

	my $mesg = $self->{ldap}->add( $user_dn, attr => [ %user_attrs ] );
	ok( !$mesg->is_error(), "Add new user" ) or diag( $mesg->error() );

	ok( $self->check_search( $user_dn, %user_attrs ), "Add new user (search)" );
	ok( $self->check_user_mailbox( \%domain_attrs, \%user_attrs ), "Add new user (cyradm user mailbox)" );

	return;
}

sub modify_user : Test(9)
{
	my $self = shift;

	my %domain_attrs = ( dc => 'modify-user.test' );
	my %domain2_attrs = ( dc => 'modify-user-2.test' );
	my %user_attrs = ( uid => $self->{test_user}->[1] );
	my $domain_dn = $self->add_domain_entry( %domain_attrs );
	my $domain2_dn = $self->add_domain_entry( %domain2_attrs );
	my $user_dn = $self->add_user_entry( %user_attrs );

	push @{$user_attrs{mailLocalAddress}}, 'tram@modify-user.test';
	my $mesg = $self->{ldap}->modify( $user_dn, add => { mailLocalAddress => ['tram@modify-user.test'] } );
	ok( !$mesg->is_error(), "Modify user (add mailLocalAddress)" ) or diag( $mesg->error() );

	ok( $self->check_search( $user_dn, %user_attrs ), "Modify user (add mailLocalAddress) (search)" );
	ok( $self->check_user_mailbox( \%domain_attrs, \%user_attrs ), "Modify user (add mailLocalAddress) (cyradm)" );

	pop @{$user_attrs{mailLocalAddress}};
	$mesg = $self->{ldap}->modify( $user_dn, delete => { mailLocalAddress => ['tram@modify-user.test'] } );
	ok( !$mesg->is_error(), "Modify user (delete mailLocalAddress)" ) or diag( $mesg->error() );

	ok( $self->check_search( $user_dn, %user_attrs ), "Modify user (delete mailLocalAddress) (search)" );
	ok( $self->check_user_mailbox( \%domain_attrs, \%user_attrs ), "Modify user (delete mailLocalAddress) (cyradm)" );

	pop @{$user_attrs{mailLocalAddress}};
	push @{$user_attrs{mailLocalAddress}}, 'tram@modify-user-2.test';
	$mesg = $self->{ldap}->modify( $user_dn, replace => { mailLocalAddress => ['tram@modify-user-2.test'] } );
	ok( !$mesg->is_error(), "Modify user (replace mailLocalAddress)" ) or diag( $mesg->error() );

	ok( $self->check_search( $user_dn, %user_attrs ), "Modify user (replace mailLocalAddress) (search)" );
	ok( $self->check_user_mailbox( \%domain_attrs, \%user_attrs ), "Modify user (replace mailLocalAddress) (cyradm)" );

	return;
}

sub delete_user : Test(3)
{
	my $self = shift;

	my %domain_attrs = ( dc => 'delete-user.test' );
	my %user_attrs = ( uid => $self->{test_user}->[1], mailLocalAddress => ['tram@delete-user.test'] );
	my $domain_dn = $self->add_domain_entry( %domain_attrs );
	my $user_dn = $self->add_user_entry( %user_attrs );

	my $mesg = $self->{ldap}->delete( $user_dn );
	ok( !$mesg->is_error(), "Delete user" ) or diag( $mesg->error() );

	ok( !$self->check_search( $user_dn ), "Delete user (search)" );
	ok( $self->check_user_mailbox( \%domain_attrs, \%user_attrs ), "Delete user (cyradm user mailbox)" );

	return;
}

1;

=head1 SEE ALSO

L<Slapd::Backend::Cyrus::User>, L<cyradm>, L<slapd-perl>, L<slapd>

=head1 AUTHOR

Aleksey Mashanov <aleksey.mashanov@gmail.com>

=head1 LICENSE

This library is free software, you can redistribute it and/or modify it under
the same terms as Perl itself.

=cut
